%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% File name: GenerateIndices.m
%
% Description: This m-file generates a matrix of power indices
%              that is used for looping in the generation of the 
%              RHS of the ODE.
%
% Inputs: polynomialOrders - The highest order of a state variable or
%                            input component in the RHS of the ODE
%         notUsedIndices - A matrix indicating indices not used  
%                    
% Outputs: allIndices - Matrix of power combinations.
%
% Author and copyright: Torbjrn Wigren
%                       Systems and Control
%                       Department of Information Technology
%                       Uppsala University
%                       Uppsala
%                       Sweden
%
% Revision log: Created 2003-05-21.
%
% Firm revision: revision 1.0, 2004-12-23.
%					  revision 2.0, 2005-08-24 (no changes).
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [allIndices]=GenerateIndices(polynomialOrders,notUsedIndices)

% 
% Set up variables
%

allIndices=[];
maxPowerIndex=polynomialOrders';
indexLength=size(maxPowerIndex,2);
thisIndex=zeros(1,indexLength);
changePointer=indexLength+1;
notUsedIndicesNumber=size(notUsedIndices,1);

%
% Loop
%

while any(thisIndex<maxPowerIndex) % continue until second final index 
   
   % check if present combination is in notUsedIndices
    
   acceptable=1;
   for i1=1:notUsedIndicesNumber
      if all(notUsedIndices(i1,:)==thisIndex)
         acceptable=0;
      end
   end
   if acceptable
      allIndices=[allIndices; thisIndex];
   end
   
   
   % Update to next index, keeping maxorder constraints in mind
   
   changePointer=indexLength+1; % Scan from right to find first possible switch
   noIncreaseFound=1;
   while (changePointer>1 & noIncreaseFound)
      changePointer=changePointer-1;
      if thisIndex(1,changePointer)<maxPowerIndex(1,changePointer)
         noIncreaseFound=0;
         increasePointer=changePointer;
      end
   end
   
   if ~noIncreaseFound % allowed to update thisIndex?
      thisIndex(1,increasePointer)=thisIndex(1,increasePointer)+1;
      for i2=increasePointer+1:indexLength % collapse the index components to the right of increasePointer to zero!
         thisIndex(1,i2)=0;
      end
   end   
end

% Then take the final index

% check if present combination is in notUsedIndices
   
acceptable=1;
for i1=1:notUsedIndicesNumber
   if all(notUsedIndices(i1,:)==thisIndex)
      acceptable=0;
   end
end
if acceptable
   allIndices=[allIndices; thisIndex];
end

